require 'spec_helper'

describe FerrisBueller::Client do
  it "must be created with a URL" do
    expect { FerrisBueller::Client.new }.to raise_error ArgumentError, ":url is required."
  end

  it "must be created with an authentication token" do
    expect { FerrisBueller::Client.new(url: '') }.to raise_error ArgumentError, ":token is required."
  end

  it "sets the authorization header on the connection with the provided token" do
    client = FerrisBueller::Client.new(url: '', token: '1337807')
    expect(client.connection.headers['Authorization']).to eql("Token token=\"1337807\"")
  end

  context "#validate_event_attributes" do
    let(:client) { client = FerrisBueller::Client.new(url: 'fake_url', token: '1337807') }

    it "doesn't raise given an event with string keys" do
      event_attributes = {
        'event' => {
          'title' => 'test',
          'venue' => 'test',
          'shows' => 'test',
          'source_listing' => 'test',
          'source' => 'test'
        }
      }
      expect { client.validate_event_attributes(event_attributes) }.to_not raise_error
    end
  end

  context "#create_event" do
    let(:client) { client = FerrisBueller::Client.new(url: 'fake_url', token: '1337807') }

    it "POSTs the event data to the events endpoint" do
      expect(client.connection).to receive(:post).with('/api/events', anything)

      client.create_event(event_data)
    end

    it "POSTs the provided event data" do
      expect(client.connection).to receive(:post).with(anything, event_data)

      client.create_event(event_data)
    end

    it "raises an error when the event is missing from the event data" do
      event = event_data

      event.delete(:event)
      allow(client.connection).to receive(:post)
      expect { client.create_event(event) }.to raise_error ArgumentError, ":event is required."
    end

    it "raises an error if any of the event attributes are missing" do
      allow(client.connection).to receive(:post)

      event_data[:event].keys.each do |key|
        key_event_data = event_data
        key_event_data[:event].delete key

        expect { client.create_event(key_event_data) }.to raise_error ArgumentError, ":event => #{key.inspect} is required."
      end
    end
  end

  def event_data
    {
      event: {
        title: "bueller_test_title",
        venue: {
          name: "bueller_test_venue_name",
          locality: "bueller_test_locality",
          region: "bueller_test_region"
        },
        shows: [
          {
            date: Time.now.to_s,
            date_description: "bueller_test_show_date_description",
            time_description: "bueller_test_show_date_description"
          }
        ],
        source_listing: {
          title: "bueller_test_source_listing_title"
        },
        source: "csv_upload"
      }
    }
  end
end
